<?php

namespace Drupal\markdown_exporter\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Component\Utility\Crypt;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\ReplaceCommand;

/**
 * Configure JSON:API settings for this site.
 *
 * @internal
 */
class ContentTypeFieldMappingSettings extends ConfigFormBase {

  /**
   * Default fields for generating markdown
   * TODO
   */
  // protected const DEFAULTFIELDS = [
  //   'type' => $this->t('Content type'), 
  //   'title' => $this->t('Title'), 
  //   'created' => $this->t('Created date')
  // ];

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'markdown_exporter_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['markdown_exporter.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    
    /* Retrieve non editable config */
    $markdown_exporter_config = $this->config('markdown_exporter.settings');


    $entity_field_json = $markdown_exporter_config->get('markdown_exporter_fields');
    $content_types = \Drupal::entityTypeManager()
      ->getStorage('node_type')
      ->loadMultiple();
    $types = [];
    foreach ($content_types as $key => $val) {
      $types[$val->id()] = $val->label();
    }
    $entityKey = \Drupal::request()->query->get('entity_type');
    $entity_type_selected = (isset($entityKey) && !empty($entityKey)) ? $entityKey : '';

    /* creating form */
    $form['info'] = [
      '#markup' => t('<h1 style="color: #212121;">Content Type Field Mapping</h1><p style="color: #3E2723;">You can choose a Content Type and map the fields to the markdown file. The choices you make in this section will generate a JSON configuration that you can manually modify from the setting tab.</p><p>Detailed instruction will be updated soon.</p>')
    ];
    $form['entity_type'] = array(
      '#type' => 'select',
      '#title' => t('Select Content Types'),
      '#default_value' => $entity_type_selected,
      '#options' => array_merge(array(t('-- Select --')), $types),
      '#description' => t('Select content types get field list.'),
      '#ajax' => [
        // Could also use [get_class($this), 'listContentTypeFields'].
        'callback' => '::listContentTypeFields',
        'wrapper' => 'content-type-field-wrapper',
      ],
    );
    // Add a wrapper that can be replaced with new HTML by the ajax callback.
    // This is given the ID that was passed to the ajax callback in the '#ajax'
    // element above.
    $form['content_type_field_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'content-type-field-wrapper'],
    ];

    // Add a field element.
    $entityType = $form_state->getValue('entity_type');
    if (!empty($entityType)) {
      // Formatter fields
      $form['content_type_field_wrapper']['formatters']= array(
        '#type' => 'details',
        '#title' => $this->t('Formatter Fields'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE
      );
      $form['content_type_field_wrapper']['formatters']['formatter_fields'] = [
        '#type' => 'checkboxes',
        '#multiple' =>  true,
        '#options' => $this->getContentTypeFields($entityType),
      ];

      //Content fields
      $form['content_type_field_wrapper']['bodyfields']= array(
        '#type' => 'details',
        '#title' => $this->t('Body Fields'),
        '#collapsible' => TRUE,
        '#collapsed' => TRUE
      );
      $form['content_type_field_wrapper']['bodyfields']['content_fields'] = [
        '#type' => 'checkboxes',
        '#multiple' =>  true,
        '#options' => $this->getContentTypeFields($entityType),
      ];
    }

    return parent::buildForm($form, $form_state);

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm (array &$form, FormStateInterface $form_state) {
    $markdown_exporter_config = $this->config('markdown_exporter.settings')->get('markdown_exporter_fields');
    $entityFields = (!empty($markdown_exporter_config)) ? (array)json_decode($markdown_exporter_config) : array();
    $entityType = $form_state->getValue('entity_type');

    $formatterFields = array_values(array_filter($form_state->getValue('formatter_fields')));
    $contentFields = array_values(array_filter($form_state->getValue('content_fields')));
    
    if(!empty($formatterFields) || !empty($contentFields)) {
      $entityFields[$entityType]->formatter = $formatterFields;
      $entityFields[$entityType]->content =   $contentFields;
    } else {
      unset($entityFields[$entityType]);
    }
    // echo '<pre>'; print_r(Json::encode($entityFields)); exit;

    $this->config('markdown_exporter.settings')
      ->set('markdown_exporter_fields', Json::encode($entityFields));

    parent::submitForm($form, $form_state);
  }

  /**
   * Ajax callback for the color dropdown.
   */
  public function listContentTypeFields (array $form, FormStateInterface $form_state) {
    return $form['content_type_field_wrapper'];
  }

  /**
   * Return fields under a content type
   *
   * @param string $entityType
   * @return array
   */
  protected function getContentTypeFields($entityType) {
    $bundleFields = [];
    $defaultFields = [
      'type' => $this->t('Content type'), 
      'title' => $this->t('Title'), 
      'created' => $this->t('Created date')
    ];
    $entity_type_id = 'node';
    foreach (\Drupal::entityManager()->getFieldDefinitions($entity_type_id, $entityType) as $field_name => $field_definition) {
      if (!empty($field_definition->getTargetBundle())) {
        // $bundleFields[$field_definition->getType()] = $field_definition->getLabel();
        $bundleFields[$field_name] = $field_definition->getLabel();
      }
    }
    $fieldArr = \array_merge($defaultFields, $bundleFields);

    return $fieldArr;
  }

}