<?php

namespace Drupal\markdown_exporter\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Component\Utility\Crypt;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\ReplaceCommand;

/**
 * Class GitHubAuthSettings
 * 
 * Configure GitHub Auth settings for this site.
 *
 * @internal
 */
class GitHubAuthSettings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'markdown_exporter_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['markdown_exporter.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    
    // Retrieve non editable config
    $markdown_exporter_config = $this->config('markdown_exporter.settings');
    
    // if  github application oauth token 
    // is not found then display a message
    if($markdown_exporter_config->get('markdown_file_commit', FALSE) && !$markdown_exporter_config->get('github_oauth_app_clientid', FALSE)) {
      \Drupal::messenger()->addMessage(
        'GitHub OAuth application token missing.
         Please get a token from <a href="
         https://github.com/settings/applications/new">
         https://github.com/settings/applications/new</a>.',
          'status'
      );
    }

    // adding form elements
    $form['info'] = [
      '#markup' => t('<h1 style="color: #212121;">GitHub Application OAuth Token</h1><p style="color: #3E2723;">Please ensure you use the following configurations when registering the OAuth application:<br><ul><li>Homepage URL: <strong>' .  $GLOBALS['base_url'] . '</strong></li><li>Authorization callback URL: <strong>' .  $GLOBALS['base_url'] . '/admin/config/markdownexporter</strong></li></ul></p>')
    ];
    $form['token'] = [  
      '#type' => 'textfield',  
      '#title' => t('GitHub OAuth Token Client ID'),
      '#required' => true,
      '#default_value' => $markdown_exporter_config->get('github_oauth_app_clientid', FALSE),		
      '#description' => t('GitHub Client ID')
    ];
    $form['secret'] = [  
      '#type' => 'textfield',  
      '#title' => t('GitHub OAuth Token Client Secret'),
      '#required' => true,
      '#default_value' => $markdown_exporter_config->get('github_oauth_app_clientsecret', FALSE),		
      '#description' => t('GitHub Client Secret'),
      '#suffix' => '<br>'
    ];

    // once the tokens are provided, 
    // display the next steps
    if ($markdown_exporter_config->get('github_oauth_app_clientid') && $markdown_exporter_config->get('github_oauth_app_clientsecret')) {
      // generate a random state 
      // string for github
      $markdown_exporter_config
        ->set('github_state_var', Crypt::hashBase64(time()))
        ->save();
      // url to authenticate with github
      $url = 'https://github.com/login/oauth/authorize?client_id=' .
       $markdown_exporter_config->get('github_oauth_app_clientid') .
        '&scope=repo&state=' .
         $markdown_exporter_config->get('github_state_var');

      // github authorization link
      $form['githubAuthorize'] = [
        '#markup' => '<a href="'.Url::fromUri($url)->toString().'">'.t('Authorize GitHub').'</a>',
        '#suffix' => '<br><br><br>',
      ];

      // if github auth token 
      // is present, display 
      // the next steps
      if ($markdown_exporter_config->get('github_access_token')) {
        //  Saving GitHub repo in config
        //  for the first time
        //  to retrieve it from next time onwards        
        if(!$markdown_exporter_config->get('github_repo_list')) {
          // fetch repository list
          $repoList = \Drupal::service('markdown_exporter.marekdown_helper')->getJsonResponse('user/repos');
          \Drupal::messenger()->addMessage(t('Repository list fetched from GitHub.'), 'status');
          // store list in a variable 
          // to reduce future api calls
          $repoArr = [];
          foreach ($repoList as $val) {
            $repoArr[$val['full_name']] = $val['name'];
          }
          $markdown_exporter_config
            ->set('github_repo_list', $repoArr)
            ->save();
        }
        // building the remaining form
        $form['next_info_2'] = [
          '#markup' => t('<h2 style="color: #212121;">Select Repository</h2><p style="color: #3E2723;">Select the repository you wish to push the generated markdown files to.</p>')
        ];
        $pushToRepo = $markdown_exporter_config->get('github_pushto_repo');
        $form['select_repository'] = array(
          '#type' => 'select',
          '#title' => t('Select Repository'),
          '#default_value' => $pushToRepo ? $pushToRepo : false,
          '#options' => array_merge(array(t('-- Select --')), $markdown_exporter_config->get('github_repo_list')),
          '#description' => t('Select Repository get field list.'),
          '#ajax' => [
            // Could also use [get_class($this), 'fetchRepoDirs'].
            'callback' => '::fetchRepoDirs',
            'wrapper' => 'repo-field-wrapper',
          ],
        );

        // Add a wrapper that can be replaced with new HTML by the ajax callback.
        // This is given the ID that was passed to the ajax callback in the '#ajax'
        // element above.
        $form['repo_field_wrapper'] = [
          '#type' => 'container',
          '#attributes' => ['id' => 'repo-field-wrapper'],
        ];
        // Add a field element.
        $selectRepository = $form_state->getValue('select_repository');
        $savedDirecory = $markdown_exporter_config->get('github_pushto_repo_dir');
        $savedBranch = $markdown_exporter_config->get('github_pushto_repo_branch');
        
        if (!empty($selectRepository)) {
          $dirList = $this->getRepositoryDirBranches($selectRepository, 'DIR');
          $branchList = $this->getRepositoryDirBranches($selectRepository, 'BRC');
          $form['repo_field_wrapper']['select_repository_dir'] = [
            '#type' => 'select',
            '#title' => $this->t('Select Repository Directory'),
            '#description' => $this->t('Select Repository Directory'),
            '#options' => $dirList,
          ];
          $form['repo_field_wrapper']['select_repository_branches'] = [
            '#type' => 'select',
            '#title' => $this->t('Select Repository Branch'),
            '#description' => $this->t('Select Repository Branch'),
            '#options' => $branchList,
          ];
          if ($selectRepository == $pushToRepo) {
            $form['repo_field_wrapper']['select_repository_dir']['#default_value'] = $savedDirecory;
            $form['repo_field_wrapper']['select_repository_branches']['#default_value'] = $savedBranch;
          }
        }
        
      }
    }
    
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('markdown_exporter.settings')
      ->set('github_oauth_app_clientid', $form_state->getValue('token'))
      ->set('github_oauth_app_clientsecret', $form_state->getValue('secret'))
      ->set('markdown_file_overwrite', $form_state->getValue('markdown_file_overwrite'))
      ->set('markdown_file_commit', $form_state->getValue('markdown_file_commit'))
      ->set('github_pushto_repo', $form_state->getValue('select_repository'))
      ->set('github_pushto_repo_dir', $form_state->getValue('select_repository_dir'))
      // ->set('github_pushto_repo_dir', 'public')
      ->set('github_pushto_repo_branch', $form_state->getValue('select_repository_branches'))
      // ->set('github_pushto_repo_branch', 'master')
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Ajax callback for the repository dropdown.
   */
  public function fetchRepoDirs (array $form, FormStateInterface $form_state) {
    return $form['repo_field_wrapper'];
  }

  /**
   * Return directory and branches
   * for selected repository
   *
   * @param string $selectRepository
   * @param string $type
   * @return array
   */
  protected function getRepositoryDirBranches($selectRepository, $type = "") {
    /* fetch the object tree list from github */
    if ($type == "") {
      return;
    }

    $optionsListArr = [];

    // call getJsonResponse helper service to retrieve details
    // DIR: represents directory to fetch for repository
    // BRC: stands for branch
    // getJsonResponse() returns only branches whose protected != 1
    switch ($type) {
      case 'DIR' :
        $url_frag = 'repos/' . $selectRepository . '/contents/';
        $resp = \Drupal::service('markdown_exporter.marekdown_helper')->getJsonResponse($url_frag);
        /* format into dir list only */
        $optionsListArr['<root>'] = '<root>';
        foreach($resp as $key=>$val) {
          if($val['type'] == 'dir'){
            $optionsListArr[$val['path']] = $val['name'];
          }
        }
        break;
      case 'BRC' :
        $url_brac = 'repos/' . $selectRepository . '/branches';
        $branches = \Drupal::service('markdown_exporter.marekdown_helper')->getJsonResponse($url_brac);
        foreach ($branches as $k=>$v) {
          if ($v['protected'] != 1) {
            $optionsListArr[$v['name']] = $v['name'];
          }
        }
        break;
    }

    return $optionsListArr;    
  }
}
