<?php

namespace Drupal\markdown_exporter\Helper;

use Drupal\Component\Serialization\Json;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Url;
use Markdownify\Parser;
use Markdownify\Converter;

/**
 * Class MarkdownGenerationHelper.
 *
 * @package Drupal\markdown_exporter\Helper
 */
class MarkdownGenerationHelper {
  
  /**
   * GitHub API URL
   */
  private $githubApiEndpoint = "https://api.github.com/";

  /**
   * Helper reusuable function
   * Generate markdown content
   *
	 * Converts the content of the node to markdown
	 * @param string $url_alias
	 * @param Drupal\Core\Entity\EntityInterface $node
	 * @param array $entity_fields
   */
	public function  mdContentGenerate ($url_alias, $node, $entity_fields) {
		$mdObj = new Converter(0, MDFY_BODYWIDTH, MDFY_KEEPHTML);
		// formatting the alias
		$url_alias_formatted = strpos($url_alias, '/') ? str_replace('/', "-", $url_alias) : $url_alias;
			
		// Content file in .md format
		$file_name = $url_alias_formatted . '.md';
		$bundle = $node->bundle();
		
		if (array_key_exists($bundle, $entity_fields) && !empty($entity_fields[$bundle])) {
			
			$formatter_fields = $entity_fields[$bundle]['formatter'];
			array_unshift($formatter_fields, "type", "title", "created");
			$content_fields = $entity_fields[$bundle]['content'];
			// if (!array_key_exists('body', $entity_fields[$bundle]['content'])) {
			// 	$content_fields = array_unshift($content_fields, 'body');
			// }
		} else {
			$formatter_fields = array('type', 'title', 'created');
			$content_fields = array('body');
		}
		
		// Preparing MD formatter
		$formatter =  "---\n";
		$formatter .=  "uri: " . $url_alias . "\n";
		$i=0;
		foreach($formatter_fields as $fld1) {
			$field_value = '';
			$formatter_field_data = '';
			
			switch($fld1) {
				case 'type':
					$field_value = $fld1 . ": " . $node->bundle() . "\n";
					break;
				case 'title':
					$field_value = $fld1 . ": " . $node->title->value . "\n";
					break;
				case 'created':
					$field_value = $fld1 . ": " . $node->created->value . "\n";
					break;
				case 'author':
					$uid = $node->getOwnerId();
					$user = \Drupal\user\Entity\User::load($uid);
					if (!is_null($user)) {
						$field_value = $fld1 . ": " . $user->getDisplayName() . "\n";
					}
					break;
				default:
					$customFieldVal = $node->get($fld1)->getValue();
					
					if(isset($customFieldVal)) {
						if (count($customFieldVal) > 1) {
							$field_value .= $fld1 . ":\n";
							foreach($customFieldVal as $valC) {
								foreach($valC as $k=>$v) {
									if (!is_array($v)) {
										$field_value .= "\t" . $k . ":" . $v . "\n";
									}
								}
							}
						} else {
							$formatter_field_data = $node->get($fld1)->getValue();
							if(!is_array($formatter_field_data)) {  
								$field_value = $fld1 . ": " . $formatter_field_data . "\n";
							} else {
								if(isset($formatter_field_data[$node->language][0]['value'])) { 
									$field_value = $fld1 . ": " . $formatter_field_data[$node->language][0]['value'] . "\n";
								} else {
									if(
										isset(
											$formatter_field_data[$node->language][0]['uri']) &&
												!empty(
													$formatter_field_data[$node->language][0]['uri']
												)
										){ // for image field
										$field_value = $fld1 . ": " . file_create_url($formatter_field_data[$node->language][0]['uri']) . "\n";
									}
								}
							}
						}								
					}
			}
			$formatter .=  $field_value;
		}
		$formatter .=  "---\n\n";

		// Preparing MD content
		$mdcontent = '';
		if (is_array($content_fields) && !empty($content_fields)) {
			foreach($content_fields as $fld2) {
				if(isset($node->{$fld2})) {
					$content_field_data = $node->get($fld2)->getValue();
					$mdcontent .= $mdObj->parseString($content_field_data[0]['value'])."\r\n";
				}  
			}
		}
		// Markdown file content
		$contents = $formatter . $mdcontent;

		return $contents;
  }

  /**
   * GitHub Response
   *
   * @param string $url
   * @return Json
   */
  public function getJsonResponse($url) {
    $url = $this->githubApiEndpoint . $url;
    $headers = [
      'Accept' => 'application/json', 
      'Authorization' => 'token ' . \Drupal::config('markdown_exporter.settings')->get('github_access_token')
    ];

    $result = \Drupal::httpClient()->get($url, [
      'headers' => $headers,
    ]);
    $resp = $result->getBody()->getContents();
    $response = Json::decode($resp);
    
    return $response;
  }

  /**
   * Push Markdown File to Git
   * Uses GITAPI V3
   *
   * @return void
   */
  public function pushToGitRepo ($fileName = "", $fileObj = "", $id = "") {

    if ($fileName == "") {
      return;
    }
    
    /* Prepare to push to the file to GitHub */
		$path = \Drupal::config('markdown_exporter.settings')->get('github_pushto_repo_dir') == '<root>' ? '' : \Drupal::config('markdown_exporter.settings')->get('github_pushto_repo_dir');
    $token = \Drupal::config('markdown_exporter.settings')->get('github_access_token');
    $repo = \Drupal::config('markdown_exporter.settings')->get('github_pushto_repo');

    // fileName comes with prevailing "/" 
		$apiUrl = $this->githubApiEndpoint . 'repos/' . $repo . '/contents/'.$path.$fileName;   
    $authKey = 'token ' . $token;
		$fileInfo = NULL;
		$response = NULL;
		$sha = true;
    try {
      $fileInfo = \Drupal::httpClient()->get($apiUrl, [
        'headers' => [
          'Content-Type' => 'application/json',
          'Authorization' => $authKey,
        ],
        // 'request.options' => [
        //    'http_errors' => false,
        // ]
			]);
    } catch (\GuzzleHttp\Exception\RequestException $e) {
			$resp =$e->getResponse();
			if ($resp->getStatusCode() == 404) {
				$sha = false;
			}
			$resp =$e->getResponse();
			\Drupal::messenger()->addMessage($resp->getBody()->getContents(), 'error');
			\Drupal::messenger()->addMessage($resp->getStatusCode(), 'error');
			\Drupal::logger('markdown_exporter')->error('Error in committing file with response : %resp.',
			array(
					'%resp' => Json::encode($resp->getBody()->getContents()),
			));
			\Drupal::logger('markdown_exporter')->error('Full Guzzle response : %resp.',
			array(
					'%resp' => Json::encode($resp),
			));
    } catch (\GuzzleHttp\Exception\ServerException $e) {
			$req = $e->getRequest();
			$resp =$e->getResponse();
			\Drupal::messenger()->addMessage($resp->getBody()->getContents(), 'error');
			\Drupal::messenger()->addMessage($resp->getStatusCode(), 'error');
			\Drupal::logger('markdown_exporter')->error('Error in committing file with response : %resp.',
			array(
					'%resp' => Json::encode($resp->getBody()->getContents()),
			));
			\Drupal::logger('markdown_exporter')->error('Full Guzzle response : %resp.',
			array(
					'%resp' => Json::encode($resp),
			));
    }
    catch (\GuzzleHttp\Exception\BadResponseException $e) {
			$req = $e->getRequest();
			$resp =$e->getResponse();
			\Drupal::messenger()->addMessage($resp->getBody()->getContents(), 'error');
			\Drupal::messenger()->addMessage($resp->getStatusCode(), 'error');
			\Drupal::logger('markdown_exporter')->error('Error in committing file with response : %resp.',
			array(
					'%resp' => Json::encode($resp->getBody()->getContents()),
			));
			\Drupal::logger('markdown_exporter')->error('Full Guzzle response : %resp.',
			array(
					'%resp' => Json::encode($resp),
			));
    }
    catch(Exception $e){
			\Drupal::messenger()->addMessage($e, 'error');
		}

		$data = [];
		if ($sha) {
			$response = $fileInfo->getBody()->getContents();
			$data = Json::decode($response);
		}
		$contents = \file_get_contents($fileObj->getFileUri());
		$content = \base64_encode($contents);
		$stream_wrapper_manager = \Drupal::service('stream_wrapper_manager')->getViaUri($fileObj->getFileUri());
		$file_path = $stream_wrapper_manager->realpath();
		$hash = \hash_file('md5', $file_path);
		
		/* Commit Message for file */
		$message = 'Markdown export for node ' . $id . ' committed via GitHub API on ' . date('Y-m-d');
		$committer = [];
		$user = \Drupal::currentUser();
		$committer['name'] = $user->getDisplayName();
		$committer['email'] = $user->getEmail();
		$params = array(
			'message' => $message,
			'committer' => $committer,
			'content' => $content,
		);

		/* Rely on fileInfo Hash if file not found */
		if (!empty($data)) {
			$params['sha'] = (!$sha && $data['sha'] != $hash) ? $hash : $data['sha'];
		}
		if (\Drupal::config('markdown_exporter.settings')->get('github_pushto_repo_branch')) {
			$params['branch'] = \Drupal::config('markdown_exporter.settings')->get('github_pushto_repo_branch');
		}
		
		/**
		 * SHA created with retrieved data / file hash
		 * PUT request to place for content api
		 * to push the file to GIT
		 */
		$result = NULL;
		try{
			$result = \Drupal::httpClient()->put($apiUrl, [
				'json' => $params,
				'headers' => [
					'Content-Type' => 'application/json',
					'Authorization' => $authKey,
				],
				// 'http_errors' => FALSE,
			]);
		} catch (\GuzzleHttp\Exception\RequestException $e) {
			$req = $e->getRequest();
			$resp =$e->getResponse();
			\Drupal::messenger()->addMessage($resp->getBody()->getContents(), 'error');
			\Drupal::messenger()->addMessage($resp->getStatusCode(), 'error');
			\Drupal::logger('markdown_exporter')->error('Error in committing file with response : %resp.',
			array(
					'%resp' => Json::encode($resp->getBody()->getContents()),
			));
			\Drupal::logger('markdown_exporter')->error('Full Guzzle response : %resp.',
			array(
					'%resp' => Json::encode($resp),
			));
		}
		catch (\GuzzleHttp\Exception\ServerException $e) {
			$req = $e->getRequest();
			\Drupal::messenger()->addMessage($resp->getBody()->getContents(), 'error');
			\Drupal::logger('markdown_exporter')->error('Error in committing file with response : %resp.',
			array(
					'%resp' => Json::encode($resp->getBody()->getContents()),
			));
			\Drupal::logger('markdown_exporter')->error('Full Guzzle response : %resp.',
			array(
					'%resp' => Json::encode($resp),
			));
		}
		catch (\GuzzleHttp\Exception\BadResponseException $e) {
			$req = $e->getRequest();
			$resp =$e->getResponse();
			\Drupal::messenger()->addMessage($resp->getBody()->getContents(), 'error');
			\Drupal::logger('markdown_exporter')->error('Error in committing file with response : %resp.',
			array(
					'%resp' => Json::encode($resp->getBody()->getContents()),
			));
			\Drupal::logger('markdown_exporter')->error('Full Guzzle response : %resp.',
			array(
					'%resp' => Json::encode($resp),
			));
		}
		catch(Exception $e){
			\Drupal::messenger()->addMessage($e, 'error');
		}
		
		if ($result) {
			$resp = $result->getBody()->getContents();
			/* Drupal watchdog log */
			\Drupal::logger('markdown_exporter')->notice('@type: commited with response : %resp.',
			array(
					'@type' => 'markdown_exporter commit',
					'%resp' => $resp,
			));
			\Drupal::messenger()->addMessage(t('File commited with response: %resp', ['%resp' => $resp]), 'status');

			return;
		}
    \Drupal::messenger()->addMessage(t("Unexpected error committing file, contact administrator!"), 'error');

    return;
  }
}
